#include <M5Stack.h>

#include "SOFT_SPI.h"
#include "MFRC522S.h"

#include "efont.h"               // Unicode表示用ライブラリ
#include "efontESP32.h"          // Unicode表示用ライブラリ　
#include "efontEnableJa.h"       // Unicode表示用ライブラリ（日本語）

// PINSET 
#define CLK_PIN  16    // Arduino 13 / M5Stack Basic 16
#define MOSI_PIN 17    // Arduino 11 / M5Stack Basic 17
#define MISO_PIN 2     // Arduino 12 / M5Stack Basic 2
#define SS_PIN   5     // Arduino 10 / M5Stack Basic 5
#define RST_PIN  26    // Arduino  9 / M5Stack Basic 26

MFRC522 mfrc522(SS_PIN, RST_PIN);   // Create MFRC522 instance

char strBuff[80] ;
int selNo = 0 ;
int selNum = 4 ;
static struct Data_t {
  char IDNo[16] ;
  char NameE[32] ;
  char NameJ[32] ;
} data[] = { "0001","SASARA SATO","さとう ささら",
             "0002","TUDUMI SUZUKI","すずき つづみ",
             "0003","AMATO TAKAHASHI","タカハシ アマト",
             "","","" } ;

bool flagWriteMode = false ;
bool flagDispMenu = false ;

//*****************************************************************************************//
void setup() {
  M5.begin();        // Init M5Core.

  M5.Power.begin();  // Init Power module.
  M5.Speaker.setVolume(3);

  M5.Lcd.clear(BLACK);
  M5.Lcd.setTextFont(4);
  M5.Lcd.setTextSize(1);
  
  SSPI.begin(CLK_PIN,MISO_PIN,MOSI_PIN); // Init Software SPI

  mfrc522.PCD_Init();                    // Init MFRC522 card
  dispMenu( ) ;
}

//*****************************************************************************************//
void loop() {
  M5.update();
  if (M5.BtnB.wasPressed()==1) {
    flagDispMenu = flagDispMenu ? false : true ;
    dispMenu( ) ;
    return ;
  }

  if (flagDispMenu) {
    if (M5.BtnA.wasPressed()==1) {
      selNo = (selNo + 1) % selNum ;
      dispMenu( ) ;
      return ;
    }
  
    if (M5.BtnC.wasPressed()==1) {
      flagWriteMode = flagWriteMode ? false : true ;
      dispMenu( ) ;
      return ;
    }
  }

  if (flagWriteMode) {
    cardWrite(selNo) ;
  } else {
    cardRead( );
  }
}

//*****************************************************************************************//
void dispMenu( ) {
  M5.Lcd.setTextSize(1);
  M5.Lcd.clear(BLACK);
  M5.Lcd.setCursor(0,0);
  M5.Lcd.setTextColor(BLUE);
  M5.Lcd.print("DATA WRITE to REF CARD.");
  M5.Lcd.drawLine(0, 24, 319, 24 , WHITE);

  M5.Lcd.drawLine(0, 166, 319, 166 , WHITE);
  if (flagDispMenu) {
    M5.Lcd.setCursor(30,210);
    M5.Lcd.setTextColor(GREEN);
    M5.Lcd.print("[SEL]");
    M5.Lcd.setCursor(200,210);
    M5.Lcd.setTextColor((flagWriteMode ? MAGENTA : DARKGREY));
    M5.Lcd.print("[WRITE]");

    M5.Lcd.setCursor(0,170);
    M5.Lcd.setTextColor(DARKGREY,BLACK);
    sprintf(strBuff,"%s %s",data[selNo].IDNo,data[selNo].NameJ) ;
    printEfont(strBuff,0,170,2);
  }
}

void dispReadError( ) {
  M5.Lcd.fillRect(0, 25, 320, 141, BLACK);

  M5.Lcd.setTextSize(1);
  M5.Lcd.setTextColor(RED,BLACK);
  M5.Lcd.setCursor(10,60);
  M5.Lcd.print("ERROR.");
  playTone(1) ;
  delay(1000) ;
}

void dispCardData(char *buff1,char *buff2,char *buff3) {
  M5.Lcd.setTextSize(1);
  M5.Lcd.setTextColor(WHITE,BLACK);
  M5.Lcd.setCursor(0,60);
  M5.Lcd.println(F("No:"));
  M5.Lcd.setCursor(0,95);
  M5.Lcd.println(F("E :"));
  M5.Lcd.setCursor(0,130);
  M5.Lcd.println(F("J :"));

  printEfont(buff1,45,50,2);
  printEfont(buff2,45,85,2);
  printEfont(buff3,45,120,2);
}

void printHex(byte *buffer, byte bufferSize) {
  for (byte i = 0; i < bufferSize; i++) {
    M5.Lcd.print(buffer[i] < 0x10 ? " 0" : " ");
    M5.Lcd.print(buffer[i], HEX);
  }
}

void dispCardID(byte *buffer, byte bufferSize) {
  M5.Lcd.setTextSize(1);
  M5.Lcd.setTextColor(WHITE,BLACK);
  M5.Lcd.setCursor(0,26);
  M5.Lcd.print(F("ID:"));
  printHex(buffer,bufferSize) ;
  M5.Lcd.println();
}

void playTone(int playNo) {
  switch(playNo) {
    case 0:
      M5.Speaker.tone(659, 200);
      delay(200);
      M5.Speaker.tone(523, 200);
      break ;

    case 1:
      M5.Speaker.tone(440, 100);
      delay(100);
      M5.Speaker.mute();
      delay(100);
      M5.Speaker.tone(440, 100);
      delay(100);
      M5.Speaker.mute();
      delay(100);
      M5.Speaker.tone(440, 100);
      break ;
  }
  M5.Speaker.mute();
}

//*****************************************************************************************//
void cardRead( ) {
  // Prepare key - all keys are set to FFFFFFFFFFFFh at chip delivery from the factory.
  MFRC522::MIFARE_Key key;
  for (byte i = 0; i < 6; i++) key.keyByte[i] = 0xFF;

  //some variables we need
  byte block;
  byte len;
  MFRC522::StatusCode status;

  //-------------------------------------------
  // Reset the loop if no new card present on the sensor/reader. This saves the entire process when idle.
  if ( ! mfrc522.PICC_IsNewCardPresent()) {
    return ;
  }

  // Select one of the cards
  if ( ! mfrc522.PICC_ReadCardSerial()) {
    return ;
  }
  
  M5.Lcd.fillRect(0, 25, 320, 141, BLACK);

  // Check is the PICC of Classic MIFARE type
  MFRC522::PICC_Type piccType = mfrc522.PICC_GetType(mfrc522.uid.sak);
  if (piccType != MFRC522::PICC_TYPE_MIFARE_MINI &&  
    piccType != MFRC522::PICC_TYPE_MIFARE_1K &&
    piccType != MFRC522::PICC_TYPE_MIFARE_4K) {
    dispCardID(mfrc522.uid.uidByte, mfrc522.uid.size);
    M5.Lcd.println(F("Not MIFARE Classic."));
    playTone(1) ;
    delay(3000) ; // コレは表示の見栄えを良くするためのdelay、なくても良い。
    return ;
  }

  //------------------------------------------- GET DATA
  byte buffer[32];
  byte buffer1[32];
  byte buffer2[32];
  byte buffer3[32];
  memset((char *)buffer1,0,32) ;
  memset((char *)buffer2,0,32) ;
  memset((char *)buffer3,0,32) ;

  //------------------------------------------- GET NO
  block = 4;
  len = 32;
  status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, block, &key, &(mfrc522.uid));
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
//    dispReadError() ;
    return ;
  }

  status = mfrc522.MIFARE_Read(block, buffer, &len);
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
//    dispReadError() ;
    return ;
  }
  memcpy(buffer1,buffer,16) ;

  //---------------------------------------- GET NAME E
  block = 5 ;
  len = 32;
  status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, block, &key, &(mfrc522.uid));
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
//    dispReadError() ;
    return ;
  }

  status = mfrc522.MIFARE_Read(block, buffer, &len);
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
//    dispReadError() ;
    return ;
  }
  memcpy(buffer2,buffer,16) ;

  block = 6 ;
  len = 32;
  status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, block, &key, &(mfrc522.uid));
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
//    dispReadError() ;
    return ;
  }

  status = mfrc522.MIFARE_Read(block, buffer, &len);
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
//    dispReadError() ;
    return ;
  }
  memcpy(&buffer2[16],buffer,16) ;

  //---------------------------------------- GET NAME J
  block = 8 ;
  len = 32;
  status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, block, &key, &(mfrc522.uid));
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
//    dispReadError() ;
    return ;
  }

  status = mfrc522.MIFARE_Read(block, buffer, &len);
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
//    dispReadError() ;
    return ;
  }
  memcpy(buffer3,buffer,16) ;

  block = 9 ;
  len = 32;
  status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, block, &key, &(mfrc522.uid));
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
//    dispReadError() ;
    return ;
  }

  status = mfrc522.MIFARE_Read(block, buffer, &len);
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
//    dispReadError() ;
    return ;
  }
  memcpy(&buffer3[16],buffer,16) ;

  mfrc522.PICC_HaltA();
  mfrc522.PCD_StopCrypto1();

  //----------------------------------------
  dispCardID(mfrc522.uid.uidByte, mfrc522.uid.size);
  dispCardData((char *)buffer1,(char *)buffer2,(char *)buffer3) ;
  playTone(0) ;
  delay(1000); //change value if you want to read cards faster
}

//*****************************************************************************************//
void cardWrite(int dataNO) {
  // Prepare key - all keys are set to FFFFFFFFFFFFh at chip delivery from the factory.
  MFRC522::MIFARE_Key key;
  for (byte i = 0; i < 6; i++) key.keyByte[i] = 0xFF;

  // Reset the loop if no new card present on the sensor/reader. This saves the entire process when idle.
  if ( ! mfrc522.PICC_IsNewCardPresent()) {
    return;
  }

  // Select one of the cards
  if ( ! mfrc522.PICC_ReadCardSerial()) {
    return;
  }

  M5.Lcd.fillRect(0, 25, 320, 141, BLACK);
  dispCardID(mfrc522.uid.uidByte, mfrc522.uid.size);
  M5.Lcd.setTextColor(RED);
  sprintf(strBuff,"DATA WRITE - No.%s",data[dataNO].IDNo) ;
  M5.Lcd.print(strBuff);

  MFRC522::PICC_Type piccType = mfrc522.PICC_GetType(mfrc522.uid.sak);

  byte *buffer ;
  byte block;
  MFRC522::StatusCode status;
  byte len;

  buffer = (byte *)data[dataNO].IDNo ;
  block = 4;
  status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, block, &key, &(mfrc522.uid));
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
    dispReadError() ;
    return;
  }

  // Write block
  status = mfrc522.MIFARE_Write(block, buffer, 16);
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
    dispReadError() ;
    return;
  }

  buffer = (byte *)data[dataNO].NameE ;
  block = 5;
  status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, block, &key, &(mfrc522.uid));
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
    dispReadError() ;
    return;
  }

  // Write block
  status = mfrc522.MIFARE_Write(block, buffer, 16);
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
    dispReadError() ;
    return;
  }

  block = 6;
  status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, block, &key, &(mfrc522.uid));
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
    dispReadError() ;
    return;
  }

  // Write block
  status = mfrc522.MIFARE_Write(block, &buffer[16], 16);
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
    dispReadError() ;
    return;
  }

  buffer = (byte *)data[dataNO].NameJ;
  block = 8;
  status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, block, &key, &(mfrc522.uid));
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
    dispReadError() ;
    return;
  }

  // Write block
  status = mfrc522.MIFARE_Write(block, buffer, 16);
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
    dispReadError() ;
    return;
  }

  block = 9;
  status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, block, &key, &(mfrc522.uid));
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
    dispReadError() ;
    return;
  }

  // Write block
  status = mfrc522.MIFARE_Write(block, &buffer[16], 16);
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
    dispReadError() ;
    return;
  }

  mfrc522.PICC_HaltA(); // Halt PICC
  mfrc522.PCD_StopCrypto1();  // Stop encryption on PCD

  flagWriteMode = false ;
  playTone(0) ;
  delay(1000) ;

  dispMenu( ) ;
}
//*****************************************************************************************//
