#include <M5Stack.h>

#include "SOFT_SPI.h"
#include "MFRC522S.h"

// PINSET 
#define CLK_PIN  16    // Arduino 13 / M5Stack Basic 16
#define MOSI_PIN 17    // Arduino 11 / M5Stack Basic 17
#define MISO_PIN 2     // Arduino 12 / M5Stack Basic 2
#define SS_PIN   5     // Arduino 10 / M5Stack Basic 5
#define RST_PIN  26    // Arduino  9 / M5Stack Basic 26

MFRC522 rfid(SS_PIN, RST_PIN); // Instance of the class
MFRC522::MIFARE_Key key; 

// Init array that will store new NUID 
byte nuidPICC[4];
char strBuff[24] ;

void setup() { 
  M5.begin();        // Init M5Core.  初始化 M5Core

  M5.Power.begin();  // Init Power module.  初始化电源模块
  M5.Lcd.clear(BLACK);
  M5.Lcd.setTextFont(4);
  M5.Lcd.setTextSize(1);
  
  SSPI.begin(CLK_PIN,MISO_PIN,MOSI_PIN); // Init Software SPI
  rfid.PCD_Init(); // Init MFRC522 

  M5.Lcd.println("REF CARD READ.");  // Print text on the screen (string)

  for (byte i = 0; i < 6; i++) {
    key.keyByte[i] = 0xFF;
  }

//  Serial.println(F("This code scan the MIFARE Classsic NUID."));
//  Serial.print(F("Using the following key:"));
//  printHex(key.keyByte, MFRC522::MF_KEY_SIZE);
}
 
void loop() {

  // Reset the loop if no new card present on the sensor/reader. This saves the entire process when idle.
  if ( ! rfid.PICC_IsNewCardPresent())
    return;

  // Verify if the NUID has been readed
  if ( ! rfid.PICC_ReadCardSerial())
    return;

  M5.Lcd.clear(BLACK);
  M5.Lcd.setCursor(0,0);

  MFRC522::PICC_Type piccType = rfid.PICC_GetType(rfid.uid.sak);
  // Check is the PICC of Classic MIFARE type
  if (piccType != MFRC522::PICC_TYPE_MIFARE_MINI &&  
    piccType != MFRC522::PICC_TYPE_MIFARE_1K &&
    piccType != MFRC522::PICC_TYPE_MIFARE_4K) {
    M5.Lcd.println(F("Not MIFARE Classic."));
    M5.Lcd.print(F("ID:"));
    printHex(rfid.uid.uidByte, rfid.uid.size);
    M5.Lcd.println();
    delay(3000) ; // コレは表示の見栄えを良くするためのdelay、なくても良い。
    return;
  }

  M5.Lcd.print(F("PICC type: "));
  M5.Lcd.println(rfid.PICC_GetTypeName(piccType));

  if (rfid.uid.uidByte[0] != nuidPICC[0] || 
    rfid.uid.uidByte[1] != nuidPICC[1] || 
    rfid.uid.uidByte[2] != nuidPICC[2] || 
    rfid.uid.uidByte[3] != nuidPICC[3] ) {
    M5.Lcd.println("New Card.");

    // Store NUID into nuidPICC array
    for (byte i = 0; i < 4; i++) {
      nuidPICC[i] = rfid.uid.uidByte[i];
    }
   
    M5.Lcd.print(F("ID:"));
    printHex(rfid.uid.uidByte, rfid.uid.size);
    M5.Lcd.println();

    int pos = 0 ;
    uint8_t *buffer = rfid.uid.uidByte ;
    for (byte i = 0; i < rfid.uid.size; i++) {
      strBuff[pos++] = (((buffer[i] & 0xF0) >> 4) >= 10) ? 'A' + (((buffer[i] & 0xF0) >> 4) - 10) : '0' + ((buffer[i] & 0xF0) >> 4) ;
      strBuff[pos++] = ((buffer[i] & 0x0F) >= 10) ? 'A' + ((buffer[i] & 0x0F) - 10) : '0' + (buffer[i] & 0x0F) ;
      strBuff[pos++] = ' ' ;
    }
    strBuff[pos++] = 0 ;
  } else {
    M5.Lcd.println(F("Card read previously."));
    M5.Lcd.print(F("ID:"));
    printHex(rfid.uid.uidByte, rfid.uid.size);
    M5.Lcd.println();
  }

  // Halt PICC
  rfid.PICC_HaltA();

  // Stop encryption on PCD
  rfid.PCD_StopCrypto1();

  delay(3000) ; // コレは表示の見栄えを良くするためのdelay、なくても良い。
}


/**
 * Helper routine to dump a byte array as hex values to Serial. 
 */
void printHex(byte *buffer, byte bufferSize) {
  for (byte i = 0; i < bufferSize; i++) {
    M5.Lcd.print(buffer[i] < 0x10 ? " 0" : " ");
    M5.Lcd.print(buffer[i], HEX);
  }
}
