#include <M5Stack.h>
#include <SD.h>

#include "SOFT_SPI.h"
#include "MFRC522S.h"

#include "efont.h"               // Unicode表示用ライブラリ
#include "efontESP32.h"          // Unicode表示用ライブラリ　
#include "efontEnableJa.h"       // Unicode表示用ライブラリ（日本語）

// PINSET 
#define CLK_PIN  16    // Arduino 13 / M5Stack Basic 16
#define MOSI_PIN 17    // Arduino 11 / M5Stack Basic 17
#define MISO_PIN 2     // Arduino 12 / M5Stack Basic 2
#define SS_PIN   5     // Arduino 10 / M5Stack Basic 5
#define RST_PIN  26    // Arduino  9 / M5Stack Basic 26
#define SD_SEL_PIN 4   //            / M5Stack Basic 4

MFRC522 mfrc522(SS_PIN, RST_PIN);   // Create MFRC522 instance
bool flagDisplay = false ;
unsigned long dispTime;

//*****************************************************************************************//
void setup() {
  M5.begin();        // Init M5Core.

  M5.Power.begin();  // Init Power module.
  M5.Speaker.setVolume(3);
  M5.Lcd.clear(BLACK);
  M5.Lcd.setTextFont(4);
  M5.Lcd.setTextSize(1);

  if(!SD.begin(SD_SEL_PIN)){
    M5.Lcd.setTextSize(1);
    M5.Lcd.clear(BLACK);
    M5.Lcd.setCursor(0,0);
    M5.Lcd.setTextColor(RED);
    M5.Lcd.print("SD Card Mount Failed.");
    return ;
  }
  
  SSPI.begin(CLK_PIN,MISO_PIN,MOSI_PIN); // Init Software SPI

  mfrc522.PCD_Init();                    // Init MFRC522 card
  dispMenu( ) ;
}

//*****************************************************************************************//
void loop() {
  cardRead( );
  if (flagDisplay && (millis() - dispTime > 5000)) {
    dispMenu( ) ;
  }
}

//*****************************************************************************************//
void dispMenu( ) {
  M5.Lcd.setTextSize(1);
  M5.Lcd.clear(BLACK);
  M5.Lcd.setCursor(10,0);
  M5.Lcd.setTextColor(WHITE);
  M5.Lcd.print("REF ID CARD CHECK.");
  M5.Lcd.drawLine(0, 24, 319, 24 , WHITE);
  M5.Lcd.drawLine(0, 208, 319, 208 , WHITE);
  M5.Lcd.drawJpgFile(SD, "/0000.jpg", 10 , 30 );
  flagDisplay = false ;
}

void dispReadError( ) {
  M5.Lcd.setTextColor(RED,BLACK);
  printEfont("ERROR",180,30,3);
  printEfont("所属不明",180,85,2);
}

void cardError( ) {
  M5.Lcd.fillRect(0, 25, 320, 183 , BLACK);
  M5.Lcd.setTextColor(RED,BLACK);
  printEfont("CARD",180,30,3);
  printEfont("ERROR",180,85,3);
  M5.Lcd.drawJpgFile(SD, "/0000.jpg", 10 , 30 );
  playTone(1) ;
  delay(1000) ;
  dispTime = millis();
  flagDisplay = true ;
}

void dispCardData(char *buff1,char *buff2,char *buff3) {
  M5.Lcd.fillRect(0, 25, 320, 183 , BLACK);
  M5.Lcd.setTextSize(2);
  M5.Lcd.setTextSize(1);
  M5.Lcd.setTextColor(WHITE,BLACK);
  char strBuff[80] ;
  sprintf(strBuff,"/%4s.jpg",buff1) ;

  if (('0' <= buff1[0] <= '9') && 
      ('0' <= buff1[1] <= '9') &&
      ('0' <= buff1[2] <= '9') &&
      ('0' <= buff1[3] <= '9') &&
      buff1[4] == 0) {
    printEfont(buff1,180,30,3);
    printEfont(buff2,180,80,1);
    printEfont(buff3,10,165,2);
  
    if (SD.exists(strBuff)) {
      M5.Lcd.drawJpgFile(SD, strBuff, 10 , 30 );
    } else {
      M5.Lcd.drawJpgFile(SD, "/0000.jpg", 10 , 30 );
    }
    playTone(0) ;
  } else {
    M5.Lcd.drawJpgFile(SD, "/0000.jpg", 10 , 30 );
    dispReadError() ;
    playTone(1) ;
  }
  delay(1000);
  dispTime = millis();
  flagDisplay = true ;
}

void printHex(byte *buffer, byte bufferSize) {
  for (byte i = 0; i < bufferSize; i++) {
    M5.Lcd.print(buffer[i] < 0x10 ? " 0" : " ");
    M5.Lcd.print(buffer[i], HEX);
  }
}

void dispCardID(byte *buffer, byte bufferSize) {
  M5.Lcd.fillRect(0, 209, 320, 31 , BLACK);
  M5.Lcd.setTextSize(1);
  M5.Lcd.setTextColor(WHITE,BLACK);
  M5.Lcd.setCursor(0,210);
  M5.Lcd.print(F("UID:"));
  printHex(buffer,bufferSize) ;
  M5.Lcd.println();
}

void playTone(int playNo) {
  switch(playNo) {
    case 0:
      M5.Speaker.tone(659, 200);
      delay(200);
      M5.Speaker.tone(523, 200);
      break ;

    case 1:
      M5.Speaker.tone(440, 100);
      delay(100);
      M5.Speaker.mute();
      delay(100);
      M5.Speaker.tone(440, 100);
      delay(100);
      M5.Speaker.mute();
      delay(100);
      M5.Speaker.tone(440, 100);
      break ;
  }
  M5.Speaker.mute();
}

//*****************************************************************************************//
void cardRead( ) {
  // Prepare key - all keys are set to FFFFFFFFFFFFh at chip delivery from the factory.
  MFRC522::MIFARE_Key key;
  for (byte i = 0; i < 6; i++) key.keyByte[i] = 0xFF;

  //some variables we need
  byte block;
  byte len;
  MFRC522::StatusCode status;

  //-------------------------------------------
  // Reset the loop if no new card present on the sensor/reader. This saves the entire process when idle.
  if ( ! mfrc522.PICC_IsNewCardPresent()) {
    return ;
  }

  // Select one of the cards
  if ( ! mfrc522.PICC_ReadCardSerial()) {
    return ;
  }
  
  M5.Lcd.fillRect(0, 25, 320, 183 , BLACK);

  // Check is the PICC of Classic MIFARE type
  MFRC522::PICC_Type piccType = mfrc522.PICC_GetType(mfrc522.uid.sak);
  if (piccType != MFRC522::PICC_TYPE_MIFARE_MINI &&  
    piccType != MFRC522::PICC_TYPE_MIFARE_1K &&
    piccType != MFRC522::PICC_TYPE_MIFARE_4K) {
    dispCardID(mfrc522.uid.uidByte, mfrc522.uid.size);
    cardError( );
//    M5.Lcd.println(F("Not MIFARE Classic."));
//    playTone(1) ;
//    delay(3000) ; // コレは表示の見栄えを良くするためのdelay、なくても良い。
    return ;
  }

  //------------------------------------------- GET DATA
  byte buffer[32];
  byte buffer1[32];
  byte buffer2[32];
  byte buffer3[32];
  memset((char *)buffer1,0,32) ;
  memset((char *)buffer2,0,32) ;
  memset((char *)buffer3,0,32) ;

  //------------------------------------------- GET NO
  block = 4;
  len = 32;
  status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, block, &key, &(mfrc522.uid));
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
    return ;
  }

  status = mfrc522.MIFARE_Read(block, buffer, &len);
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
    return ;
  }
  memcpy(buffer1,buffer,16) ;

  //---------------------------------------- GET NAME E
  block = 5 ;
  len = 32;
  status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, block, &key, &(mfrc522.uid));
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
    return ;
  }

  status = mfrc522.MIFARE_Read(block, buffer, &len);
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
    return ;
  }
  memcpy(buffer2,buffer,16) ;

  block = 6 ;
  len = 32;
  status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, block, &key, &(mfrc522.uid));
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
    return ;
  }

  status = mfrc522.MIFARE_Read(block, buffer, &len);
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
    return ;
  }
  memcpy(&buffer2[16],buffer,16) ;

  //---------------------------------------- GET NAME J
  block = 8 ;
  len = 32;
  status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, block, &key, &(mfrc522.uid));
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
    return ;
  }

  status = mfrc522.MIFARE_Read(block, buffer, &len);
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
    return ;
  }
  memcpy(buffer3,buffer,16) ;

  block = 9 ;
  len = 32;
  status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, block, &key, &(mfrc522.uid));
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
    return ;
  }

  status = mfrc522.MIFARE_Read(block, buffer, &len);
  if (status != MFRC522::STATUS_OK) {
    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
    return ;
  }
  memcpy(&buffer3[16],buffer,16) ;

  mfrc522.PICC_HaltA();
  mfrc522.PCD_StopCrypto1();

  //----------------------------------------
  dispCardID(mfrc522.uid.uidByte, mfrc522.uid.size);
  dispCardData((char *)buffer1,(char *)buffer2,(char *)buffer3) ;
}

//*****************************************************************************************//
